using System;
using System.Data;
using System.Data.OleDb;

using Banco.Interfaces;

namespace Banco.OleDbImpl
{
	/// <summary>
	/// Summary description for DAL.
	/// </summary>
	public class OleDbProvider : IBanco
	{
		//private const string CONN_STR = @"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=..\..\contas.mdb;User ID=;Password=";
        private string CONN_STR
        {
            get
            {
                //para usar app.config da maneira Visual Studio 2003
                //return System.Configuration.ConfigurationManager.AppSettings["ConnStr"];

                // usando modleo de properties/settings do VS 2005
                return Properties.Settings.Default.ConnStr;
            }
        }

		#region Acesso directo  BD

		public DataSet GetContas()
		{
			OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM contas", CONN_STR);
			DataSet ds = new DataSet();
			da.Fill(ds, "contas");
			return ds;
		}

		public decimal GetSaldo(string conta)
		{
			// TO DO
			return 0;
		}

		public DataSet GetMovimentos(string conta)
		{
			// TO DO
			return null;
		}

		public DataSet GetMovimentos(int conta)
		{
			OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM movimentos WHERE conta=" + conta.ToString(), CONN_STR);
			DataSet ds = new DataSet();
			da.Fill(ds, "movimentos");
			return ds;
		}

		public decimal AddMovimento(string conta, DateTime dt, decimal montante)
		{
			// TO DO
			return 0;
		}

		public decimal AddMovimento(int conta, DateTime dt, decimal montante)
		{
			OleDbConnection cnx = OpenConnection();
			OleDbTransaction tx = cnx.BeginTransaction();

			OleDbCommand cmd = new OleDbCommand("INSERT INTO movimentos (data, montante, conta, xxx) VALUES(?,?,?, ?)", cnx);
			cmd.Transaction = tx;
			OleDbParameter p = cmd.Parameters.Add("data", OleDbType.Date);
			p.Value = dt;
			//cmd.Parameters.Add("mont", montante);
			p = cmd.Parameters.Add("mont", OleDbType.Currency);
			p.Value = montante;
			cmd.Parameters.AddWithValue("conta", conta);


			p = cmd.Parameters.Add("xxx", OleDbType.Decimal);
			p.Precision = 28;
			p.Scale = 4;
			p.Value = montante;

			cmd.ExecuteNonQuery();

			OleDbCommand cmd2 = new OleDbCommand("UPDATE contas SET saldo=saldo+? WHERE id=?", cnx);
			cmd2.Transaction = tx;
			//cmd2.Parameters.Add("mont", montante);
			p = cmd2.Parameters.Add("mont", OleDbType.Currency);
			p.Value = montante;
			cmd2.Parameters.AddWithValue("conta", conta);
			cmd2.ExecuteNonQuery();

			OleDbCommand cmd3 = new OleDbCommand("SELECT saldo FROM contas WHERE id=?", cnx);
			cmd3.Transaction = tx;
			cmd3.Parameters.AddWithValue("conta", conta);
			decimal s = (decimal)cmd3.ExecuteScalar();

			tx.Commit();
			cnx.Close();

			return s;
		}
		
		#endregion

		#region Typed DataSet

		public BDContas GetConta(int conta)
		{
			BDContas ds = new BDContas();
			OleDbConnection cnx = OpenConnection();
			//esta no  a maneira mais eficiente de ir buscar os dados j que vai necessitar de efectuar 2 SELECT
			//adicionalmente deveria limitar o numero de movimentos que traz
			OleDbDataAdapter da1 = new OleDbDataAdapter("SELECT * FROM contas WHERE id=" + conta.ToString(), cnx);
			OleDbDataAdapter da2 = new OleDbDataAdapter("SELECT * FROM movimentos WHERE conta=" + conta.ToString(), cnx);
			da1.Fill(ds, "contas");
			da2.Fill(ds, "movimentos");
			cnx.Close();
			return ds;
		}

		public BDContas GetConta(string numeroConta)
		{
			BDContas ds = new BDContas();
			OleDbConnection cnx = OpenConnection();
			//esta no  a maneira mais eficiente de ir buscar os dados j que vai necessitar de efectuar 3 SELECT
			//adicionalmente deveria limitar o numero de movimentos que traz
			OleDbDataAdapter da1 = new OleDbDataAdapter("SELECT * FROM contas WHERE numero='" + numeroConta + "'", cnx);
			OleDbDataAdapter da2 = new OleDbDataAdapter("SELECT * FROM movimentos WHERE conta=(SELECT id FROM contas WHERE numero = '" + numeroConta + "')", cnx);
			da1.Fill(ds, "contas");
			da2.Fill(ds, "movimentos");
			cnx.Close();
			return ds;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="dsConta"></param>
		public void UpdateConta(BDContas dsConta)
		{
			//
			//apenas vai considerar os novos movimentos e a actualizao do saldo  calculada
			//

			//obter linhas adicionadas
			DataSet ds = dsConta.GetChanges(DataRowState.Added);
			if (ds == null)
				return;

			//calcular diferencial do saldo
			decimal s = 0;
			foreach (BDContas.movimentosRow r in ds.Tables["movimentos"].Rows)
				s += r.montante;
			
			//abrir conexo e iniciar transaco
			OleDbConnection cnx = OpenConnection();
			OleDbTransaction tx = cnx.BeginTransaction();

			//construir DataAdpter e definir comandos SQL auxiliares vi OleDbCommandBuilder
			OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM movimentos", cnx);
			da.SelectCommand.Transaction = tx;
			OleDbCommandBuilder cb = new OleDbCommandBuilder(da);

			// actualziar tabela de movimentos
			da.Update(ds, "movimentos");
			
			//actualizar saldo da conta
			OleDbCommand cmd2 = new OleDbCommand("UPDATE contas SET saldo=saldo+? WHERE id=?", cnx);
			cmd2.Transaction = tx;
			cmd2.Parameters.AddWithValue("mont", s);
			cmd2.Parameters.AddWithValue("conta", dsConta.contas[0].ID);
			cmd2.ExecuteNonQuery();

			//terminar transaco e fechar conexo
			tx.Commit();
			cnx.Close();
		}
#endregion

		#region Mtodos auxiliares

		private OleDbConnection OpenConnection()
		{
			OleDbConnection conn = new OleDbConnection(CONN_STR);
			conn.Open();
			return conn;
		}

		#endregion

        public void MigrateToXml(string filename)
        {
            OleDbConnection cnx = new OleDbConnection(CONN_STR);
            try
            {
                cnx.Open();
                DataSet ds = new DataSet();
                OleDbDataAdapter da = new OleDbDataAdapter("SELECT * FROM Contas", cnx);
                da.Fill(ds, "contas");
                da = new OleDbDataAdapter("SELECT * FROM Movimentos", cnx);
                da.Fill(ds, "movimentos");

                ds.Relations.Add("ContaMovimentos", ds.Tables["Contas"].Columns["id"], ds.Tables["movimentos"].Columns["conta"]);
                ds.Tables["contas"].Columns["id"].AutoIncrement = true;
                ds.Tables["movimentos"].Columns["id"].AutoIncrement = true;

                ds.WriteXml(filename, XmlWriteMode.WriteSchema);
            }
            catch (OleDbException ex)
            {
                throw;
            }
            finally
            {
                if (cnx.State != ConnectionState.Closed)
                    cnx.Close();
            }
        }
	}
}
