using System;
using System.Data;

using Banco.Interfaces;

namespace Banco.XmlImpl
{
    public class XmlProvider : IBanco
    {
        private string DATA_FILE 
        {
            get 
            {
                // a la VS 2003
                //return System.Configuration.ConfigurationManager.AppSettings["DataFile"];  

                // a la VS 2005
                return Properties.Settings.Default.DataFile;
            }
        }

        private DataSet LoadData()
        {
            DataSet ds = new DataSet();
            ds.ReadXml(DATA_FILE);

            //ds.Tables["contas"].Columns["id"].AutoIncrement = true;
            //ds.Tables["movimentos"].Columns["id"].AutoIncrement = true;

            return ds;
        }

        private void SaveData(DataSet ds)
        {
            ds.WriteXml(DATA_FILE, XmlWriteMode.WriteSchema);
        }

        #region IBanco Members

        public System.Data.DataSet GetContas()
        {
            DataSet ds = LoadData();
            DataSet dsRet = new DataSet(); 
            dsRet.Tables.Add(ds.Tables["contas"].Copy());
            return dsRet;
        }

        public decimal GetSaldo(string conta)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public System.Data.DataSet GetMovimentos(string conta)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public System.Data.DataSet GetMovimentos(int conta)
        {
            DataSet ds = LoadData();
            DataSet dsRet = new DataSet();
            dsRet.Tables.Add(ds.Tables["movimentos"].Clone());
            DataRow[] res = ds.Tables["movimentos"].Select("conta=" + conta);
            foreach (DataRow r in res)
                dsRet.Tables["movimentos"].Rows.Add(r.ItemArray);
            return dsRet;
        }

        public decimal AddMovimento(string conta, DateTime dt, decimal montante)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public decimal AddMovimento(int conta, DateTime dt, decimal montante)
        {
            DataSet ds = LoadData();
            DataRow drMov = ds.Tables["movimentos"].NewRow();
            drMov["conta"] = conta;
            drMov["data"] = dt;
            drMov["montante"] = montante;
            ds.Tables["movimentos"].Rows.Add(drMov);
            DataRow drConta = ds.Tables["contas"].Select("id=" + conta)[0];
            decimal saldo = (decimal)drConta["saldo"] + montante;
            drConta["saldo"] = saldo;

            SaveData(ds);

            return saldo;
        }

        public BDContas GetConta(int conta)
        {
            DataSet ds = LoadData();
            BDContas dsRet = new BDContas();
            DataRow[] dr = ds.Tables["contas"].Select("id="+conta);
            dsRet.contas.Rows.Add(dr[0].ItemArray);
            foreach (DataRow rm in dr[0].GetChildRows("ContaMovimentos"))
                dsRet.movimentos.Rows.Add(rm.ItemArray);
            dsRet.AcceptChanges();
            return dsRet;
        }

        public BDContas GetConta(string numeroConta)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpdateConta(BDContas dsConta)
        {
            DataSet ds = LoadData();
            
            DataSet dsDif = dsConta.GetChanges(DataRowState.Deleted);
            if (dsDif.Tables["contas"].Rows.Count > 0)
                throw new InvalidOperationException("no  permitido remover contas");
            foreach (DataRow rMovimento in dsDif.Tables["movimentos"].Rows)
            {
                DataRow r = ds.Tables["movimentos"].Select("id=" + rMovimento["id", DataRowVersion.Original])[0];
                ds.Tables["movimentos"].Rows.Remove(r);
            }
            
            dsDif = dsConta.GetChanges();
            foreach (DataRow rConta in dsDif.Tables["contas"].Rows)
            {
                DataRow drContaBD=null;
                switch (rConta.RowState)
                {
                    case DataRowState.Added:
                        drContaBD = ds.Tables["contas"].Rows.Add(rConta.ItemArray);
                        break;
                    case DataRowState.Modified:
                        drContaBD = ds.Tables["contas"].Select("id=" + rConta["id"])[0];
                        drContaBD.ItemArray = rConta.ItemArray;
                        break;
                    case DataRowState.Unchanged:
                        drContaBD = ds.Tables["contas"].Select("id=" + rConta["id"])[0];
                        break;
                }

                // GetChildRows no traz DataRows apagadas
                foreach (DataRow rMovimento in rConta.GetChildRows(dsDif.Relations[0]))
                {
                    DataRow r;
                    switch (rMovimento.RowState)
                    {
                        case DataRowState.Added:
                            ds.Tables["movimentos"].Rows.Add(rMovimento.ItemArray);
                            break;
                        case DataRowState.Modified:
                            r = ds.Tables["movimentos"].Select("id=" + rMovimento["id"])[0];
                            r.ItemArray = rMovimento.ItemArray;
                            break;
                    }
                }

                decimal saldo = 0m;
                foreach (DataRow r in drContaBD.GetChildRows("ContaMovimentos"))
                    saldo += (decimal)r["montante"];
                drContaBD["saldo"] = saldo;
            }

            SaveData(ds);
        }


        #endregion
    }
}
